package a2;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Point;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

public class Line {

	final int NEAR_THRESHOLD = 20;
	private Point startPoint;
	private Point endPoint;
	private Shape theShape;
	private int thicknessIndex = 0;
	private int colourIndex = 0;

	public Line(int x1, int y1, int x2, int y2) {
		startPoint = new Point(x1, y1);
		endPoint = new Point(x2, y2);
		theShape = getShape(startPoint, endPoint);
		thicknessIndex = 0;
		colourIndex = 0;
	}

	public Point getEndPoint() {
		return endPoint;
	}

	public Point getStartPoint() {
		return startPoint;
	}

	public boolean isNearTo(Point point) {
		Rectangle2D proximalField = new Rectangle2D.Float(point.x, point.y,
				NEAR_THRESHOLD, NEAR_THRESHOLD);
		return theShape.intersects(proximalField);
	}

	/**
	 * Determine whether the passed point is proximal to this line's center
	 * segment
	 * 
	 * @param point
	 * @return
	 */
	public boolean isNearToCenterSegment(Point point) {
		// the displacement from each of the endpoints to the passed point
		// should be no less than 0.25 the length of the line plus the passed
		// point should be proximal
		boolean isProximal = isNearTo(point);
		if (!isProximal) {
			return false;
		} else {
			double lineLength = getDistance(endPoint, startPoint);
			double distToStartPoint = getDistance(startPoint, point);
			double distToEndPoint = getDistance(endPoint, point);
			boolean isFarEnoughFromStart = distToStartPoint > lineLength * 0.25;
			boolean isFarEnoughFromEnd = distToEndPoint > lineLength * 0.25;
			System.out.println(distToStartPoint + "\t" + distToEndPoint + "\t"
					+ lineLength * 0.25 + "\t" + isFarEnoughFromStart + "\t"
					+ isFarEnoughFromEnd);
			return isFarEnoughFromStart && isFarEnoughFromEnd;
		}
	}

	private double getDistance(Point p1, Point p2) {
		int xDisp = p1.x - p2.x;
		int yDisp = p1.y - p2.y;
		double lineLength = Math.sqrt(xDisp * xDisp + yDisp * yDisp);
		return lineLength;
	}

	public void displace(Point displacement) {
		startPoint = new Point(startPoint.x + displacement.x, startPoint.y
				+ displacement.y);
		endPoint = new Point(endPoint.x + displacement.x, endPoint.y
				+ displacement.y);
		theShape = getShape(startPoint, endPoint);
	}

	public String toString() {
		return "line: " + startPoint + "\t" + endPoint;
	}

	public void advanceLineThicknessToNextValue() {
		thicknessIndex = (thicknessIndex + 1)
				% LineDrawModel.THICKNESS_VALUES.length;
	}

	private Shape getShape(Point startPoint, Point endPoint) {
		Shape theShape = new Line2D.Float(startPoint, endPoint);
		// // Now we can compute the corner points...
		// int dx = thickness / 2;
		// int dy = thickness / 2;
		// int xPoints[] = new int[4];
		// int yPoints[] = new int[4];
		// xPoints[0] = startPoint.x + dx;
		// yPoints[0] = startPoint.y + dy;
		// xPoints[1] = startPoint.x - dx;
		// yPoints[1] = startPoint.y - dy;
		// xPoints[2] = endPoint.x - dx;
		// yPoints[2] = endPoint.y - dy;
		// xPoints[3] = endPoint.x + dx;
		// yPoints[3] = endPoint.y + dy;

		return theShape;
	}

	public Stroke getStroke() {
		return new BasicStroke(LineDrawModel.THICKNESS_VALUES[thicknessIndex]);
	}

	public void advanceColourToNextValue() {
		colourIndex = (colourIndex + 1) % LineDrawModel.COLOUR_VALUES.length;
	}

	public Color getColour() {
		return LineDrawModel.COLOUR_VALUES[colourIndex];
	}

}
