package a2;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * Encapsulation of LineDrawListener, taken from Olsen Fig 3.16, p. 67
 * 
 * Note: this listener is designed to listen for events that originate from the
 * model. For events originating from the model, install this listener on the
 * model. Have the model invoke the lineWillChange and modelHasChanged methods.
 * 
 * This listener is also designed to listen for Mouse and MouseMotion events.
 * Note that these two types of events are dispatched from the windowing system
 * from two different dispatch sources. Therefore, this listener should be
 * installed on both of those dispatch sources.
 * 
 * @author mb
 * 
 */
public interface LineDrawListener extends MouseListener, MouseMotionListener {

	/**
	 * Signals that a particular line will change. Assumes that the line to
	 * which index refers is within the model.
	 * 
	 * @param index
	 * @param newX1
	 * @param newY1
	 * @param newX2
	 * @param newY2
	 */
	public void lineWillChange(int index, int newX1, int newY1, int newX2,
			int newY2);

	public void modelHasChanged();

}
