package a2;

import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

public class MyLineDrawListener implements LineDrawListener {

	private Component view;
	private LineDrawModel model;
	private Point startPoint;

	private Integer currentIndex = null;

	private boolean inStateOfLineChangeInProgress = false;
	private boolean inStateOfLineCreationInProgress = false;

	public MyLineDrawListener(Component view, LineDrawModel model) {
		this.view = view;
		this.model = model;
	}

	@Override
	public void lineWillChange(int index, int newX1, int newY1, int newX2,
			int newY2) {
		Line line = model.getLine(index);
		if (line != null) {
			Rectangle oldRegion = new Rectangle(line.getStartPoint().x,
					line.getStartPoint().y, line.getEndPoint().x,
					line.getEndPoint().y);
			view.invalidate(); // need to find a way to damage a region
								// selectively, p. 70
		}
		Rectangle newRegion = new Rectangle(newX1, newY1, newX2, newY2);
		view.invalidate();// need to find a way to damage a region
		// selectively
		view.repaint();
	}

	@Override
	public void modelHasChanged() {
		view.invalidate();
		view.repaint();
	}

	// The following methods are specified in the MouseListener

	@Override
	public void mouseClicked(MouseEvent e) {
		// as an exercise, experiment with removing and adding mouse listener
		// from view - you will see that this method is invoked only when the
		// mouse listener is installed
		// System.out.println("Click on point: " + e.getPoint());
		if (model.nearestLineByIndex(e.getPoint()) != null) {
			currentIndex = model.nearestLineByIndex(e.getPoint());
			// Line nearestLine = model.nearestLine(e.getPoint());
			if (model.getLine(currentIndex).isNearToCenterSegment(e.getPoint())) {
				model.advanceLineThicknessToNextValue(currentIndex);
				// nearestLine.advanceLineThicknessToNextValue();
			} else {
				// nearestLine.advanceColourToNextValue();
				model.advanceLineColorToNextValue(currentIndex);
			}
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		if (model.nearestLineByIndex(e.getPoint()) != null) {
			inStateOfLineChangeInProgress = true;
			currentIndex = model.nearestLineByIndex(e.getPoint());
			System.out.println(" near to line with index: "
					+ model.nearestLineByIndex(e.getPoint()));
			startPoint = e.getPoint();
		} else {
			inStateOfLineCreationInProgress = true;
			startPoint = e.getPoint();
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (inStateOfLineChangeInProgress) {
			inStateOfLineChangeInProgress = false;
			Point displacement = new Point(e.getPoint().x - startPoint.x,
					e.getPoint().y - startPoint.y);
			model.moveLine(currentIndex, displacement);
		} else if (inStateOfLineCreationInProgress) {
			model.addLine(startPoint.x, startPoint.y, e.getPoint().x,
					e.getPoint().y);
		}

	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseDragged(MouseEvent e) {
	}

	// The following methods are specified in the MouseMotionListener

	@Override
	public void mouseMoved(MouseEvent e) {

		// as an exercise, experiment with removing and adding mouse motion
		// listener
		// from view - you will see that this method is invoked only when the
		// mouse motion listener is installed

		// System.out.print(e.getPoint());
		if (model.nearestLineByIndex(e.getPoint()) != null) {
			// System.out.print(" near to line with index: "
			// + model.nearestLine(e.getPoint()));
		}
		// System.out.println();

	}

}
