package a2_withMultipleViews;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseMotionListener;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;

import javax.swing.JComponent;

/**
 * Encapsulation of LineDrawView, taken from:
 * 
 * - Olsen Fig 2.5, p. 19 (called DrawView in this eg)
 * 
 * - Fig 3.18 p. 70
 * 
 * @author mb
 * 
 */
public class LineDrawView2 extends Canvas {
	private LineDrawModel myModel;

	public LineDrawView2(LineDrawModel model) {
		this.myModel = model;
		LineDrawListener myListener = new MyLineDrawListener(this, myModel);
		// to listen for event dispatched from the model, install listener on
		// the model
		myModel.addListener(myListener);
		// to listen for mouse motion events, install listener as below
		this.addMouseMotionListener(myListener);
		// to listen for mouse events (which are different that *mouse motion*
		// events, install listener as below
		this.addMouseListener(myListener);
	}

	/*
	 * Paint when the AWT tells us to...
	 */
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		float yPos = 0;
		FontRenderContext frc = g2.getFontRenderContext();
		for (int i = 0; i < myModel.getNumberOfLines(); i++) {
			AttributedString aStr = new AttributedString(myModel.getLine(i)
					.toString());
			TextLayout layout = new TextLayout(aStr.getIterator(), frc);
			yPos += layout.getDescent() + layout.getAscent();
			layout.draw(g2, 0, yPos + layout.getDescent() + layout.getAscent());
		}
	}

	public Dimension getPreferredSize() {
		return new Dimension(400, 200);
	}

}
